@@ -22,7 +22,7 @@ import tempfile
 import textwrap
 import time
 from collections.abc import Iterable, Sequence
-from typing import Any, Final, NamedTuple
+from typing import Any, Callable, Final, NamedTuple
 
 import platformdirs
 import requests
@@ -39,14 +39,24 @@ import rich.text
 
 from . import APPNAME, encode, ffmpeg
 
-# import rich.traceback
-# rich.traceback.install(console=console)
-
 logger = logging.getLogger(__name__)
 logging.getLogger("requests").propagate = False
 logging.getLogger("urllib3").propagate = False
 
-console = rich.console.Console()
+console = rich.console.Console(highlight=False)
+
+
+def hl_path(path: os.PathLike[str]) -> str:
+    pl_path = pathlib.Path(path)
+    name = "" if pl_path.is_dir() else str(pl_path.name)
+    parent = str(pl_path).removesuffix(name)
+    parent = "[magenta]" + rich.markup.escape(parent) if parent else ""
+    name = "[bright_magenta]" + rich.markup.escape(name)
+    return parent + name
+
+
+def hl_url(url: str) -> str:
+    return f"[link={url}]{url}[/link]"
 
 
 def get_stream_list(
@@ -56,11 +66,10 @@ def get_stream_list(
     deep_probe: bool = False,
 ) -> list[list[tuple[str, str]]]:
     """Make and return tuples of (key,val) pairs for each stream."""
-    outjson = ffmpeg.probe(
-        ffmpeg.format_probestring(q_tuple, True),
+    outjson = ffmpeg.ff_bin.probe_json(
+        ffmpeg.format_q_tuple(q_tuple, True),
         myfileargs,
         streamtype,
-        probetype=ffmpeg.ProbeType.RAW_STREAM,
         extraargs="-pretty",
         deep_probe=deep_probe,
     )
@@ -69,10 +78,10 @@ def get_stream_list(
         return []
     if not (allstreams := outjson.get("streams")):
         return []
-    stream_list = []
+    stream_list: list[list[tuple[str, str]]] = []
     BAD_VALS: Final[set[str | int]] = {"N/A", "unknown"}
     for s in allstreams:
-        val_list = []
+        val_list: list[tuple[str, str]] = []
         ituple = (
             ("", s, q_tuple[0], BAD_VALS),
             ("tags: ", s.get("tags", {}), q_tuple[1], BAD_VALS),
@@ -88,15 +97,6 @@ def get_stream_list(
     return stream_list
 
 
-def highlight_path(path: os.PathLike[Any]) -> str:
-    pl_path = pathlib.Path(path)
-    name = str(pl_path.name)
-    parent = str(pl_path).removesuffix(name)
-    parent = "[magenta]" + rich.markup.escape(parent) if parent else ""
-    name = "[bright_magenta]" + rich.markup.escape(name)
-    return parent + name
-
-
 class InfoKeys:
     VSTREAMS: Final = {
         "codec_name",
@@ -171,7 +171,7 @@ def print_info(fopts: encode.FileOpts, deep_probe: bool = False) -> None:
 
     def make_columns(title: str, *args: Any) -> str:
         slist = get_stream_list(*args)
-        table_list = []
+        table_list: list[rich.table.Table] = []
         for subindex, vallist in enumerate(slist):
             table = rich.table.Table(
                 title=f"{subindex}",
@@ -193,16 +193,15 @@ def print_info(fopts: encode.FileOpts, deep_probe: bool = False) -> None:
         with console.capture() as capture:
             console.print(
                 rich.columns.Columns(
-                    table_list,
-                    title=rich.text.Text(title, style="bold italic"),
+                    table_list, title=rich.text.Text(title, style="bold italic")
                 )
             )
         return capture.get()
 
-    console.print(f"file: {highlight_path(fopts.fpath)}", highlight=False)
+    console.print(f"file: {hl_path(fopts.fpath)}")
     with concurrent.futures.ThreadPoolExecutor() as executor:
         fileduration = executor.submit(
-            ffmpeg.probe,
+            ffmpeg.ff_bin.probe_val,
             "duration",
             probefargs,
             probetype=ffmpeg.ProbeType.FORMAT,
@@ -234,10 +233,15 @@ def print_info(fopts: encode.FileOpts, deep_probe: bool = False) -> None:
             deep_probe,
         )
         if (duration := fileduration.result()) is not None:
-            console.print("format_duration=[green]" + duration, highlight=False)
-        console.out(vid_fut.result(), highlight=False)
-        console.out(aud_fut.result(), highlight=False)
-        console.out(sub_fut.result(), highlight=False)
+            console.print("format_duration=[green]" + duration)
+        if vid := vid_fut.result():
+            console.out(vid)
+        if aud := aud_fut.result():
+            console.out(aud)
+        if fopts.fpath != fopts.sfpath:
+            console.print(f"subfile: {hl_path(fopts.sfpath)}")
+        if subs := sub_fut.result():
+            console.out(subs)
 
 
 def status_wait(
@@ -245,14 +249,14 @@ def status_wait(
 ) -> None:
     """Wait on remaining background processes while showing status."""
     for future in concurrent.futures.wait(futures, 0).not_done:
-        future.add_done_callback(lambda fut: fv.update_avail.set())
+        future.add_done_callback(lambda _: fv.update_avail.set())
     if not concurrent.futures.wait(futures, 0).not_done and fv.ev.verbosity == 0:
         return
     if fv.ev.verbosity > 0:
         unfinished = fv.statuses
     else:
         unfinished = [
-            s for s in fv.statuses if s.status is not encode.StatusThread.Code.FINISHED
+            s for s in fv.statuses if s.status is not encode.StatusCode.FINISHED
         ]
 
     REFRESH_PER_SEC: Final = 10
@@ -277,23 +281,19 @@ def status_wait(
             for task_id, status in zip(task_ids, unfinished):
                 fv.update_avail.clear()
                 # TODO: 3.10 match case
-                if status.status is encode.StatusThread.Code.RUNNING:
+                if status.status is encode.StatusCode.RUNNING:
                     progress.start_task(task_id)
                     completed = status.progress
-                elif status.status is encode.StatusThread.Code.FINISHED:
+                elif status.status is encode.StatusCode.FINISHED:
                     progress.stop_task(task_id)
                     completed = 1.0
-                elif status.status is encode.StatusThread.Code.FAILED:
+                elif status.status is encode.StatusCode.FAILED:
                     progress.stop_task(task_id)
                     completed = 0
                 else:
                     progress.reset(task_id, start=False)
                     completed = 0
-                progress.update(
-                    task_id,
-                    status=status.long_status,
-                    completed=completed,
-                )
+                progress.update(task_id, status=status.long_status, completed=completed)
 
         start_time = time.perf_counter()
         sleep_time = 1 / REFRESH_PER_SEC
@@ -323,6 +323,7 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
         "bound_w": fv.ev.bound_w,
         "bound_h": fv.ev.bound_h,
     }
+    logger.debug("json payload: \n%r", json_payload)
     with requests.Session() as s:
         while True:
             try:
@@ -332,7 +333,8 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
                     raise SystemExit(1)
                 elif req.status_code != 200:
                     console.print(
-                        f"API server returned error {req.status_code}: {req.reason}"
+                        f"API server returned error {req.status_code}: {req.reason}",
+                        markup=False,
                     )
                     console.print("waiting and trying again")
                     time.sleep(1)
@@ -388,7 +390,7 @@ def start_stream(fv: encode.EncodeSession) -> None:
         "[red]{task.fields[bitrate]}",
         "•",
         "[cyan]{task.fields[speed]:<6}",
-        refresh_per_second=4,
+        refresh_per_second=10 if fv.fv("f", "duration") is None else 4,
         console=console,
     ) as progress:
         assert result.stdout is not None
@@ -396,34 +398,40 @@ def start_stream(fv: encode.EncodeSession) -> None:
         if fv.ev.clip_length:
             length = ffmpeg.duration(fv.ev.clip_length)
         else:
-            length = ffmpeg.duration(fv.v("f", "duration"))
+            length = ffmpeg.duration(fv.v("f", "duration")) or 1
         if fv.ev.timestamp and not fv.ev.clip_length:
             ts_offset = ffmpeg.duration(fv.ev.timestamp)
         else:
             ts_offset = 0.0
+        if fv.ev.delay_start:
+            ts_offset -= float(fv.ev.start_delay)
 
         ts = 0.0
         ts += ts_offset
 
         def sec_to_str(seconds: float | int) -> str:
-            m, s = divmod(seconds, 60)
+            m, s = divmod(abs(seconds), 60)
             h, m = divmod(m, 60)
-            return f"{int(h):02d}:{int(m):02d}:{int(s):02d}"
+            return f"{'-' if seconds<0 else ''}{int(h):02d}:{int(m):02d}:{int(s):02d}"
 
-        length_str = sec_to_str(length)
+        length_str = "" if fv.fv("f", "duration") is None else f"/{sec_to_str(length)}"
 
         task_id = progress.add_task(
             "Encode",
             bitrate=fv.ev.ffprogress.status["bitrate"],
             speed=fv.ev.ffprogress.status["speed"],
-            timestamp=f"{sec_to_str(ts)}/{length_str}",
+            timestamp=f"{sec_to_str(ts)}{length_str}",
             start=False,
         )
-        progress.update(task_id, total=length)
-        progress.start_task(task_id)
+        if fv.fv("f", "duration") is not None:
+            progress.update(task_id, total=length)
+            progress.start_task(task_id)
+        else:
+            progress.update(task_id, total=0)
 
+        progress_loglevel = logging.INFO
         fv.ev.ffprogress.monitor_progress(
-            result, result.stdout, maxlen=50, loglevel=logging.INFO
+            result, result.stdout, maxlen=50, loglevel=progress_loglevel
         )
 
         fv.ev.ffprogress.progress_avail.wait()
@@ -434,23 +442,23 @@ def start_stream(fv: encode.EncodeSession) -> None:
                 completed=ts,
                 bitrate=fv.ev.ffprogress.status["bitrate"],
                 speed=fv.ev.ffprogress.status["speed"],
-                timestamp=f"{sec_to_str(ts)}/{length_str}",
+                timestamp=f"{sec_to_str(ts)}{length_str}",
             )
             fv.ev.ffprogress.progress_avail.wait()
             fv.ev.ffprogress.progress_avail.clear()
 
-        ts = length
         progress.update(
             task_id,
             refresh=True,
             completed=ts,
             bitrate=fv.ev.ffprogress.status["bitrate"],
             speed=fv.ev.ffprogress.status["speed"],
-            timestamp=f"{sec_to_str(ts)}/{length_str}",
+            timestamp=f"{sec_to_str(ts)}{length_str}",
         )
 
     if result.returncode != 0:
-        logger.error("\n".join(fv.ev.ffprogress.output))
+        if not logging.getLogger("").isEnabledFor(progress_loglevel):
+            logger.error("\n".join(fv.ev.ffprogress.output))
         logger.error(f"stream finished with exit code {result.returncode}")
     else:
         console.print("stream finished")
@@ -459,9 +467,9 @@ def start_stream(fv: encode.EncodeSession) -> None:
 def stream_file(fopts: encode.FileOpts, args: argparse.Namespace) -> None:
     """Manage calculating of all stream parameters."""
     with console.status("calculating stream parameters"):
-        console.print(f"starting: {highlight_path(fopts.fpath)}", highlight=False)
+        console.print(f"starting: {hl_path(fopts.fpath)}")
         fv = encode.EncodeSession(fopts, encode.StaticEncodeVars.from_args(args))
-        futures = []
+        futures: list[concurrent.futures.Future[Any]] = []
         encode.determine_timeseek(fv)
         if not fv.ev.copy_audio:
             futures.append(fv.executor.submit(encode.determine_afilters, fv))
@@ -496,7 +504,7 @@ def process_file(
     fpath: pathlib.Path, args: argparse.Namespace, stream_flist: Sequence[pathlib.Path]
 ) -> None:
     """Format input arguments needed for a file and send to output."""
-    infile_args = []
+    infile_args: list[str] = []
     if args.playlist:
         infile_args += ["-f", "concat", "-safe", "0"]
     file_string = str(fpath)
@@ -524,8 +532,12 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
         else:
             parser.error("--bluray must be used with a directory")
     elif args.files:
-        stream_flist = []
+        stream_flist: list[pathlib.Path] = []
         for path in args.files:
+            try:
+                path = path.expanduser()
+            except RuntimeError:
+                parser.error("could not resolve user home")
             if path.is_dir():
                 stream_flist += [f for f in sorted(path.iterdir()) if f.is_file()]
             elif path.is_file():
@@ -535,7 +547,10 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
         if not stream_flist:
             parser.error("must supply a directory with at least one file")
         if args.files[-1].is_file():
-            not_same = lambda f: not f.samefile(stream_flist[-1])  # noqa: E731
+            # noqa: E731
+            not_same: Callable[[pathlib.Path], bool] = lambda f: not f.samefile(
+                stream_flist[-1]
+            )
             stream_flist = (
                 list(itertools.dropwhile(not_same, stream_flist))[:-1] or stream_flist
             )
@@ -547,7 +562,7 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
                     or args.crop
                     or args.subs
                 )
-                playlist = ffmpeg.make_playlist(
+                playlist = ffmpeg.ff_bin.make_playlist(
                     stream_flist, args.tempdir, add_duration, args.deep_probe
                 )
             process_file(playlist, args, stream_flist)
@@ -561,6 +576,9 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
         pass
 
 
+ENCODE_DEFAULTS: Final = encode.StaticEncodeVars(pathlib.Path(""))
+
+
 @dataclasses.dataclass
 class DefaultConfig:
     """Holds config file values.
@@ -569,24 +587,26 @@ class DefaultConfig:
     from files.
     """
 
-    pyffserver: bool = encode.StaticEncodeVars.pyffserver
-    protocol: str = encode.StaticEncodeVars.protocol
-    vbitrate: str = encode.StaticEncodeVars.vbitrate
-    abitrate: str = encode.StaticEncodeVars.abitrate
-    astandard: str = encode.StaticEncodeVars.astandard
-    vencoder: str = encode.StaticEncodeVars.vencoder
-    endpoint: str = encode.StaticEncodeVars.endpoint
-    api_url: str = encode.StaticEncodeVars.api_url
-    api_key: str = encode.StaticEncodeVars.api_key
-    soxr: bool = encode.StaticEncodeVars.soxr
-    preset: str = encode.StaticEncodeVars.x264_preset
-    zscale: bool = encode.StaticEncodeVars.zscale
-    vulkan: bool = encode.StaticEncodeVars.vulkan
-    fdk: bool = encode.StaticEncodeVars.fdk
-    height: int = int(encode.StaticEncodeVars.target_h)
-    shader_dir: pathlib.Path = encode.StaticEncodeVars.shader_dir
+    pyffserver: bool = ENCODE_DEFAULTS.pyffserver
+    protocol: str = ENCODE_DEFAULTS.protocol
+    vbitrate: str = ENCODE_DEFAULTS.vbitrate
+    abitrate: str = ENCODE_DEFAULTS.abitrate
+    astandard: str = ENCODE_DEFAULTS.astandard
+    vencoder: str = ENCODE_DEFAULTS.vencoder
+    endpoint: str = ENCODE_DEFAULTS.endpoint
+    api_url: str = ENCODE_DEFAULTS.api_url
+    api_key: str = ENCODE_DEFAULTS.api_key
+    soxr: bool = ENCODE_DEFAULTS.soxr
+    preset: str = ENCODE_DEFAULTS.x264_preset
+    zscale: bool = ENCODE_DEFAULTS.zscale
+    vulkan: bool = ENCODE_DEFAULTS.vulkan
+    trust_vulkan: bool = ENCODE_DEFAULTS.trust_vulkan
+    vulkan_device: int = ENCODE_DEFAULTS.vulkan_device
+    fdk: bool = ENCODE_DEFAULTS.fdk
+    hwaccel: bool = ENCODE_DEFAULTS.hwaccel
+    height: int = int(ENCODE_DEFAULTS.target_h)
     shader_list: list[str] = dataclasses.field(default_factory=list)
-    kf_target_sec: float = encode.StaticEncodeVars.kf_target_sec
+    kf_target_sec: float = ENCODE_DEFAULTS.kf_target_sec
     ffmpeg_bin: str = ffmpeg.ff_bin.ffmpeg
     ffprobe_bin: str = ffmpeg.ff_bin.ffprobe
     env: dict[str, str] = dataclasses.field(
@@ -594,7 +614,7 @@ class DefaultConfig:
     )
 
 
-def set_console_logger(verbosity: int) -> None:
+def set_console_logger(verbosity: int, logfile: pathlib.Path | None) -> None:
     """Set loglevel."""
     root_logger = logging.getLogger("")
     if verbosity >= 2:
@@ -604,15 +624,26 @@ def set_console_logger(verbosity: int) -> None:
     else:
         root_logger.setLevel(logging.WARNING)
 
-    rhandler = rich.logging.RichHandler(
-        console=console, show_time=False, show_path=False
+    handlers: list[logging.Handler] = []
+    handlers.append(
+        rich.logging.RichHandler(console=console, show_time=False, show_path=False)
     )
-    que: queue.Queue[Any] = queue.Queue()
+    if logfile is not None:
+        handlers.append(logging.FileHandler(logfile, mode="x", encoding="utf-8"))
+    que: queue.Queue[object] = queue.Queue()
     queue_handler = logging.handlers.QueueHandler(que)
-    listener = logging.handlers.QueueListener(que, rhandler)
+    listener = logging.handlers.QueueListener(que, *handlers)
     root_logger.addHandler(queue_handler)
     listener.start()
-    atexit.register(lambda x: x.stop(), listener)
+    atexit.register(shutdown_logging, listener, queue_handler)
+
+
+def shutdown_logging(
+    listener: logging.handlers.QueueListener, handler: logging.handlers.QueueHandler
+) -> None:
+    root_logger = logging.getLogger("")
+    root_logger.removeHandler(handler)
+    listener.stop()
 
 
 def download_win_ffmpeg(dltype: str = "git") -> bool:
@@ -622,8 +653,8 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
     User data path is determined from platformdirs.
     """
     console.print(
-        f"Starting download of ffmpeg {dltype} from"
-        " https://github.com/BtbN/FFmpeg-Builds/releases"
+        f"Starting download of ffmpeg {dltype} from "
+        + hl_url("https://github.com/BtbN/FFmpeg-Builds/releases")
     )
     with tempfile.TemporaryDirectory(
         prefix=f"{APPNAME}-"
@@ -650,7 +681,7 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
         ff_url = ""
         dir_name = ""
         for ass in assets:
-            if (match := re.fullmatch(download_regex, ass.get("name", ""))) is not None:
+            if match := re.fullmatch(download_regex, ass.get("name", "")):
                 dir_name = match.group("dir")
                 ff_url = ass["browser_download_url"]
                 break
@@ -673,7 +704,9 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
                 console=console,
             ) as progress:
                 task_id = progress.add_task(
-                    "download", filename=ff_url.split("/")[-1], start=False
+                    "download",
+                    filename=rich.markup.escape(ff_url.split("/")[-1]),
+                    start=False,
                 )
                 response = s.get(
                     ff_url, stream=True, headers={"Cache-Control": "no-cache"}
@@ -721,8 +754,8 @@ def win_set_local_ffmpeg(dltype: str, env: dict[str, str]) -> None:
             console.print(f"ffmpeg required to run {APPNAME}")
             raise SystemExit(1)
         console.print(
-            "No local ffmpeg found. We can download from"
-            " https://github.com/BtbN/FFmpeg-Builds/releases"
+            "No local ffmpeg found. We can download from "
+            + hl_url("https://github.com/BtbN/FFmpeg-Builds/releases")
         )
         if not rich.prompt.Confirm.ask("Would you like to download?", console=console):
             console.print(f"ffmpeg required to run {APPNAME}")
@@ -734,7 +767,7 @@ def win_set_local_ffmpeg(dltype: str, env: dict[str, str]) -> None:
         ):
             logger.error(
                 "ffmpeg download failed, download manually from"
-                " https://ffmpeg.org/download.html or try again later"
+                f" {hl_url('https://ffmpeg.org/download.html')} or try again later"
             )
             raise SystemExit(1)
     ffmpeg.ff_bin = ffmpeg.FFBin(
@@ -750,7 +783,7 @@ def get_parserconfig(
     """Return parser and config used."""
     config = DefaultConfig()
 
-    conf_list: list[str | os.PathLike[Any]] = []
+    conf_list: list[pathlib.Path] = []
 
     if not reproducible:
         conf_list.append(platformdirs.site_config_path(APPNAME) / f"{APPNAME}.conf")
@@ -769,19 +802,19 @@ def get_parserconfig(
     conf_file = conf_args.config
 
     if conf_args.show_config_dirs:
-        console.print("Searched config files in this order:", highlight=False)
+        console.print("Searched config files in this order:")
         for confile in conf_list:
-            console.print(confile)
+            console.print(hl_path(confile))
         raise SystemExit(0)
 
     if conf_file is not None and conf_file.is_file():
         conf_list.append(conf_file)
 
-    converters = {
+    converters: dict[str, Callable[[str], object]] = {
         "list": lambda val: [
             line.lstrip() for line in val.splitlines() if line.strip()
         ],
-        "path": lambda val: pathlib.Path(val),  # pylint: disable=W0108
+        "path": pathlib.Path,
     }
     parsed_config = configparser.ConfigParser(converters=converters)
     read_configs = parsed_config.read(conf_list, encoding="utf-8")
@@ -801,9 +834,11 @@ def get_parserconfig(
         config.soxr = main_conf.getboolean("soxr", config.soxr)
         config.zscale = main_conf.getboolean("zscale", config.zscale)
         config.vulkan = main_conf.getboolean("vulkan", config.vulkan)
+        config.trust_vulkan = main_conf.getboolean("trust_vulkan", config.trust_vulkan)
+        config.vulkan_device = main_conf.getint("vulkan_device", config.vulkan_device)
         config.fdk = main_conf.getboolean("fdk", config.fdk)
+        config.hwaccel = main_conf.getboolean("hwaccel", config.hwaccel)
         config.height = main_conf.getint("height", config.height)
-        config.shader_dir = main_conf.getpath("shader_dir", config.shader_dir)
         config.shader_list = main_conf.getlist("shader_list", config.shader_list)
         config.kf_target_sec = main_conf.getfloat("kf_target_sec", config.kf_target_sec)
         config.ffmpeg_bin = main_conf.get("ffmpeg_bin", config.ffmpeg_bin)
@@ -813,7 +848,7 @@ def get_parserconfig(
         # use case-sensitive keys for env values
         env_config = configparser.ConfigParser()
         # https://github.com/python/mypy/issues/2427
-        env_config.optionxform = str  # type: ignore
+        env_config.optionxform = str  # type: ignore[assignment]
         env_config.read(conf_list, encoding="utf-8")
         for key in env_config["env"]:
             config.env[key] = env_config["env"][key]
@@ -896,11 +931,13 @@ def get_parserconfig(
     resolution_group = video_parser.add_mutually_exclusive_group()
 
     parser.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase verbosity level",
+        "-v", "--verbose", action="count", default=0, help="increase verbosity level"
+    )
+    parser.add_argument(
+        "--logfile",
+        type=pathlib.Path,
+        help="path to logfile to send output to",
+        metavar="FILE",
     )
     video_parser.add_argument(
         "-b",
@@ -930,7 +967,7 @@ def get_parserconfig(
         type=pathlib.Path,
         nargs="*",
         default=[],
-        metavar="INPUT_FILES",
+        metavar="FILES",
         help=(
             "list of input files and directories; if last argument is file already"
             " contained in input list, start list from that file"
@@ -1007,6 +1044,12 @@ def get_parserconfig(
     input_parser.add_argument(
         "-B", "--bluray", help="input directory is bluray", action="store_true"
     )
+    input_parser.add_argument(
+        "--hwaccel",
+        help="Attempt to automatically use hw accelerated decoding if available",
+        action=argparse.BooleanOptionalAction,
+        default=config.hwaccel,
+    )
     subtitle_parser.add_argument(
         "-e", "--subs", help="enable subtitles", action="store_true"
     )
@@ -1028,14 +1071,14 @@ def get_parserconfig(
         "--aindex",
         help="subindex of audio stream to use (default: %(default)s)",
         type=int_ge_zero,
-        default=encode.StaticEncodeVars.aindex,
+        default=ENCODE_DEFAULTS.aindex,
         metavar="N",
     )
     video_parser.add_argument(
         "--vindex",
         help="subindex of video stream to use (default: %(default)s)",
         type=int_ge_zero,
-        default=encode.StaticEncodeVars.vindex,
+        default=ENCODE_DEFAULTS.vindex,
         metavar="N",
     )
     input_parser.add_argument(
@@ -1149,6 +1192,19 @@ def get_parserconfig(
         action=argparse.BooleanOptionalAction,
         default=config.vulkan,
     )
+    video_parser.add_argument(
+        "--trust-vulkan",
+        help="whether to trust ffmpeg vulkan to do the right thing or use workarounds",
+        action=argparse.BooleanOptionalAction,
+        default=config.trust_vulkan,
+    )
+    video_parser.add_argument(
+        "--vulkan-device",
+        type=int_ge_zero,
+        metavar="NUM",
+        help="GPU device listing to use for vulkan HW context",
+        default=config.vulkan_device,
+    )
     subtitle_parser.add_argument(
         "-z",
         "--cropsecond",
@@ -1213,29 +1269,28 @@ def get_parserconfig(
     parser.add_argument(
         "--startdelay", help="delay stream start by 30 seconds", action="store_true"
     )
+    parser.add_argument(
+        "--endpad",
+        help="Pad end of stream with nothing to prevent early stream cutoff.",
+        action=argparse.BooleanOptionalAction,
+        default=encode.StaticEncodeVars.end_pad,
+    )
     parser.add_argument(
         "--tempdir",
         type=pathlib.Path,
         help="directory to use for storing temporary files",
         metavar="DIR",
     )
-    parser.add_argument(
-        "--shaderdir",
-        type=pathlib.Path,
-        help="directory for libplacebo shaders",
-        default=config.shader_dir,
-        metavar="DIR",
-    )
     parser.add_argument(
         "--shaders",
         type=str,
         help=(
-            "list of shaders to use (specify once for each shader to add) (default:"
+            "shader to use with vulkan (specify once for each shader to add) (default:"
             " %(default)s)"
         ),
         default=config.shader_list,
         action="append",
-        metavar="SHADER",
+        metavar="PATH_TO_SHADER",
     )
     ff_binary_group.add_argument(
         "--system-ffmpeg",
@@ -1275,17 +1330,24 @@ def get_parserconfig(
     if conf_file is not None and not conf_file.is_file():
         parser.error("Passed config file must be a file.")
 
-    logger.info(f"parsed configs: {read_configs!r}")
+    logger.info("parsed configs: %r", read_configs)
 
     return parser, config
 
 
+class ConfName(NamedTuple):
+    file_name: str
+    arg_name: str
+
+
 def main() -> None:
     """Process config and CLI arguments then send off for processing."""
     parser, config = get_parserconfig(False)
     args = parser.parse_args()
 
-    set_console_logger(args.verbose)
+    if args.logfile is not None and args.logfile.exists():
+        parser.error("logfile already exists")
+    set_console_logger(args.verbose, args.logfile)
 
     if not args.system_ffmpeg:
         ffmpeg.ff_bin = ffmpeg.FFBin(
@@ -1308,8 +1370,8 @@ def main() -> None:
         else:
             console.print("Cannot find ffmpeg and ffprobe utilities in path")
             console.print(
-                "Consider downloading ffmpeg from your package manager or at"
-                " https://ffmpeg.org/download.html"
+                "Consider downloading ffmpeg from your package manager or at "
+                + hl_url("https://ffmpeg.org/download.html")
             )
             raise SystemExit(1)
 
@@ -1328,26 +1390,6 @@ def main() -> None:
     elif args.x264:
         args.vencoder = "libx264"
 
-    if not args.print_info:
-        if args.vencoder not in (
-            encode.StaticEncodeVars.VIDEO_ENCODERS & ffmpeg.ff_bin.vencoders
-        ):
-            parser.error(
-                f"selected vencoder {args.vencoder!r} not supported by ffmpeg"
-                " installation"
-            )
-
-        if args.soxr and "--enable-libsoxr" not in ffmpeg.ff_bin.build_config:
-            parser.error("soxr specified, but using an ffmpeg build without support")
-
-        if args.zscale and "zscale" not in ffmpeg.ff_bin.filters:
-            parser.error("zscale specified, but using an ffmpeg build without support")
-
-        if args.fdk and "libfdk_aac" not in ffmpeg.ff_bin.aencoders:
-            parser.error(
-                "fdk encoder specified, but using an ffmpeg build without support"
-            )
-
     if args.obs:
         args.live = True
 
@@ -1364,12 +1406,53 @@ def main() -> None:
         if next((i for i in args.copy if re.search("[aA]", i)), None):
             args.copy_audio = True
 
+    if not args.print_info:
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            vencoder_future = executor.submit(
+                lambda: args.vencoder
+                in encode.StaticEncodeVars.VIDEO_ENCODERS & ffmpeg.ff_bin.vencoders
+                if not args.copy_video
+                else True
+            )
+            zscale_future = executor.submit(
+                lambda: "zscale" in ffmpeg.ff_bin.filters
+                if not args.copy_video and args.zscale
+                else True
+            )
+            soxr_future = executor.submit(
+                lambda: "--enable-libsoxr" in ffmpeg.ff_bin.build_config
+                if not args.copy_audio and args.soxr
+                else True
+            )
+            fdk_future = executor.submit(
+                lambda: "libfdk_aac" in ffmpeg.ff_bin.aencoders
+                if not args.copy_audio and args.fdk
+                else True
+            )
+            if not vencoder_future.result():
+                parser.error(
+                    f"selected vencoder {args.vencoder!r} not supported by ffmpeg"
+                    " installation"
+                )
+            if not zscale_future.result():
+                parser.error(
+                    "zscale specified, but using an ffmpeg build without support"
+                )
+            if not soxr_future.result():
+                parser.error(
+                    "soxr specified, but using an ffmpeg build without support"
+                )
+            if not fdk_future.result():
+                parser.error(
+                    "fdk encoder specified, but using an ffmpeg build without support"
+                )
+
     if args.startdelay and (args.copy_video or args.copy_audio):
         parser.error("audio/video copying cannot be used with a start delay")
     if args.startdelay and args.timestamp is not None:
         parser.error("timestamp seeking cannot be used with a start delay")
 
-    if args.eightbit and not args.hevc_nvenc:
+    if args.eightbit and args.vencoder != "hevc_nvenc":
         parser.error("8-bit HEVC encoding must be specified with HEVC")
 
     if args.eightbit:
@@ -1446,17 +1529,13 @@ def main() -> None:
         new_config.read(config_path, encoding="utf-8")
         # keep case for env section
         new_full_config = configparser.ConfigParser()
-        new_full_config.optionxform = str  # type: ignore
+        new_full_config.optionxform = str  # type: ignore[assignment]
         new_full_config.read(config_path, encoding="utf-8")
 
         if "pyffstream" not in new_config.sections():
             new_config.add_section("pyffstream")
         main_section = new_config["pyffstream"]
 
-        class ConfName(NamedTuple):
-            file_name: str
-            arg_name: str
-
         conf_names: set[ConfName] = {
             ConfName("pyffserver", "pyffserver"),
             ConfName("protocol", "protocol"),
@@ -1471,9 +1550,11 @@ def main() -> None:
             ConfName("soxr", "soxr"),
             ConfName("zscale", "zscale"),
             ConfName("vulkan", "vulkan"),
+            ConfName("trust_vulkan", "trust_vulkan"),
+            ConfName("vulkan_device", "vulkan_device"),
             ConfName("fdk", "fdk"),
+            ConfName("hwaccel", "hwaccel"),
             ConfName("height", "height"),
-            ConfName("shader_dir", "shaderdir"),
             ConfName("kf_target_sec", "keyframe_target_sec"),
         }
         for conf in conf_names:
@@ -1483,8 +1564,7 @@ def main() -> None:
                 main_section[conf.file_name] = str(getattr(args, conf.arg_name))
         new_full_config["pyffstream"] = new_config["pyffstream"]
 
-        console.print("Writing defaults to config path:")
-        console.print(config_path)
+        console.print(f"Writing defaults to config path:\n{hl_path(config_path)}")
         with config_path.open(mode="w", encoding="utf-8") as f:
             new_full_config.write(f)
         raise SystemExit(0)
