@@ -22,17 +22,44 @@ import os
 import pathlib
 import queue
 import re
-import shlex
 import socket
 import subprocess
 import threading
 import typing
 from collections.abc import Iterable, Mapping, MutableSequence, Sequence
-from typing import Any, Final, Literal, NamedTuple, TypedDict, Union, overload
+from typing import Any, AnyStr, Final, Generic, NamedTuple, TypedDict, Union, cast
+
+# if sys.version_info >= (3, 10):
+#     from typing import TypeAlias
+# else:
+#     from typing_extensions import TypeAlias
 
 logger = logging.getLogger(__name__)
 
 
+class ProbeType(enum.Enum):
+    STREAM = enum.auto()
+    TAGS = enum.auto()
+    DISPOSITION = enum.auto()
+    FORMAT = enum.auto()
+
+
+# TODO: use typing.TypeAlias in 3.10
+StreamQueryTuple = tuple[Iterable[str], Iterable[str], Iterable[str]]
+# TODO: 3.10 | union syntax
+InitTuple = Union[StreamQueryTuple, set[str]]
+
+
+class FFProbeJSON(TypedDict, total=False):
+    # https://github.com/FFmpeg/FFmpeg/blob/master/doc/ffprobe.xsd
+    streams: Sequence[Mapping[str, Any]]
+    packets: Sequence[Mapping[str, Any]]
+    library_versions: Sequence[Mapping[str, Any]]
+    side_data_list: Sequence[Any]
+    format: Mapping[str, Any]
+    program_version: Mapping[str, Any]
+
+
 @functools.total_ordering
 class FFVersion:
     """Holds a ffmpeg component version."""
@@ -43,9 +70,6 @@ class FFVersion:
         Args:
             *args: String of type ``(N.)*N`` or an int N or FFVersion to
                 be appended to the version when constructing.
-
-        Raises:
-            ValueError: Invalid type passed to the constructor.
         """
         self._version: list[int] = []
         for arg in args:
@@ -53,10 +77,8 @@ class FFVersion:
                 self._version += arg._version
             elif isinstance(arg, str):
                 self._version += map(int, arg.split("."))
-            elif isinstance(arg, int):
-                self._version.append(int(arg))
             else:
-                raise ValueError(f"Invalid arg {arg!r} passed to FFVersion")
+                self._version.append(arg)
 
     def __repr__(self) -> str:
         return ".".join(map(str, self._version))
@@ -69,10 +91,7 @@ class FFVersion:
     def __lt__(self, other: object) -> bool:
         if not isinstance(other, (FFVersion, str)):
             return NotImplemented
-        for me, them in zip(self._version, FFVersion(other)._version):
-            if me != them:
-                return me < them
-        return False
+        return self._version < FFVersion(other)._version
 
 
 class FFBanner(NamedTuple):
@@ -106,7 +125,12 @@ class FFBin:
         env (dict[str,str]): Environmental variables to use with ffmpeg.
     """
 
-    def __init__(self, ffmpeg: str, ffprobe: str, env: dict[str, str]):
+    def __init__(
+        self,
+        ffmpeg: str = "ffmpeg",
+        ffprobe: str = "ffprobe",
+        env: dict[str, str] | None = None,
+    ):
         """Inits new FFBin instance.
 
         Args:
@@ -114,9 +138,198 @@ class FFBin:
             ffprobe: Path to ffprobe binary.
             env: Environmental variables to use with ffmpeg.
         """
-        self.ffmpeg: str = ffmpeg
-        self.ffprobe: str = ffprobe
-        self.env: dict[str, str] = env
+        self.ffmpeg = ffmpeg
+        self.ffprobe = ffprobe
+        self.env = env if env is not None else os.environ.copy()
+
+    def probe_json(
+        self,
+        entries: str,
+        fileargs: str | Iterable[str] | None,
+        streamtype: str | None = None,
+        deep_probe: bool = False,
+        extraargs: str | Iterable[str] | None = None,
+    ) -> FFProbeJSON | None:
+        """Probes a media file with ffprobe and returns results.
+
+        Generic function for probing a media file for information using
+        ffmpeg's `ffprobe` utility and returning its JSON.
+
+        Args:
+            entries:
+                Argument passed to the ``-show_entries`` flag in
+                ffprobe.
+            fileargs:
+                String of the file you want to analyze. If additional
+                args are needed to specify the input, accepts a list of
+                args to pass on.
+            streamtype:
+                Optional; Argument to pass on to the ``-select_streams``
+                flag in ffprobe. Argument not passed if None.
+            deep_probe:
+                Optional; Pass extra arguments to ffprobe in order to
+                probe the file more deeply. This is useful for
+                containers that can't be lightly inspected.
+            extraargs:
+                Optional; A list of additional arguments to past to
+                ffprobe during runtime. Can be used for example to
+                request ``-sexagesimal`` formatting of duration fields.
+
+        Returns:
+            None: The query failed or returned "unknown" or "N/A".
+
+            deserialized JSON: For raw probetype, the JSON returned
+            after deserialization.
+        """
+        # fmt: off
+        probeargs = [
+            self.ffprobe,
+            *(["-analyzeduration", "100M", "-probesize", "100M"] if deep_probe else []),
+            "-v", "0",
+            "-of", "json=c=1",
+            "-noprivate",
+            *((extraargs,) if isinstance(extraargs, str) else extraargs or ()),
+            *(("-select_streams", streamtype) if streamtype is not None else ()),
+            "-show_entries", entries,
+            *((fileargs,) if isinstance(fileargs, str) else fileargs or ()),
+        ]
+        # fmt: on
+        result = subprocess.run(
+            probeargs,
+            capture_output=True,
+            env=self.env,
+            text=True,
+            encoding="utf-8",
+            check=False,
+        )
+        if result.returncode != 0:
+            return None
+        return cast(FFProbeJSON, json.loads(result.stdout))
+
+    _QUERY_PREFIX: Final = {
+        ProbeType.STREAM: "stream=",
+        ProbeType.TAGS: "stream_tags=",
+        ProbeType.DISPOSITION: "stream_disposition=",
+        ProbeType.FORMAT: "format=",
+    }
+
+    def probe_val(
+        self,
+        entries: str,
+        fileargs: str | Iterable[str] | None,
+        streamtype: str | None = None,
+        probetype: ProbeType = ProbeType.STREAM,
+        deep_probe: bool = False,
+        extraargs: str | Iterable[str] | None = None,
+    ) -> str | None:
+        """Probes a media file with ffprobe and returns results.
+
+        Generic function for probing a media file for information using
+        ffmpeg's `ffprobe` utility. Returns individual values.
+
+        Args:
+            entries:
+                Argument passed to the ``-show_entries`` flag in
+                ffprobe. If a non-raw streamtype is specified, then the
+                argument may be the type field you want to query, for
+                example the duration.
+            fileargs:
+                String of the file you want to analyze. If additional
+                args are needed to specify the input, accepts a list of
+                args to pass on.
+            streamtype:
+                Optional; Argument to pass on to the ``-select_streams``
+                flag in ffprobe. Argument not passed if None.
+            probetype:
+                Optional; One of STREAM, TAGS, DISPOSITION, FORMAT;
+                query file for metadata of selected probetype and
+                streamtype and return the requested entries of the first
+                matching stream.
+            deep_probe:
+                Optional; Pass extra arguments to ffprobe in order to
+                probe the file more deeply. This is useful for
+                containers that can't be lightly inspected.
+            extraargs:
+                Optional; A list of additional arguments to past to
+                ffprobe during runtime. Can be used for example to
+                request ``-sexagesimal`` formatting of duration fields.
+
+        Returns:
+            None: The query failed or returned "unknown" or "N/A".
+
+            str: Returns the value of the requested query.
+
+        Raises:
+            ValueError: Invalid probetype was passed.
+        """
+        jsonout = self.probe_json(
+            self._QUERY_PREFIX[probetype] + entries,
+            fileargs,
+            streamtype,
+            deep_probe,
+            extraargs,
+        )
+        if jsonout is None:
+            return None
+        try:
+            # TODO: change to match case in 3.10
+            if probetype is ProbeType.STREAM:
+                returnval = jsonout["streams"][0][entries]
+            elif probetype is ProbeType.TAGS:
+                returnval = jsonout["streams"][0]["tags"][entries]
+            elif probetype is ProbeType.DISPOSITION:
+                returnval = jsonout["streams"][0]["disposition"][entries]
+            elif probetype is ProbeType.FORMAT:
+                returnval = jsonout["format"][entries]
+            else:
+                raise ValueError("invalid probe type query")
+        except (KeyError, IndexError):
+            # TODO: remove parentheses in 3.10
+            return None
+        return None if returnval in {"N/A", "unknown"} else str(returnval)
+
+    def make_playlist(
+        self,
+        pathlist: Iterable[pathlib.Path],
+        directory: pathlib.Path,
+        add_duration: bool,
+        deep_probe: bool = False,
+        name: str = "streamplaylist.txt",
+    ) -> pathlib.Path:
+        """Construct ffconcat playlist from path list.
+
+        Paths created are absolute, and the files are probed in parallel
+        if needed to determine the duration.
+
+        Args:
+            pathlist: Ordered pathlist to construct ffconcat with.
+            directory: Directory to put the constructed playlist in.
+            add_duration: Whether or not to include duration metadata.
+            deep_probe: Whether to probe file deeply.
+            name: Optional; name of playlist file.
+
+        Returns:
+            Path to created ffconcat playlist.
+        """
+        durfutures = None
+        if add_duration:
+            probe_args = [
+                ("duration", str(fpath), None, ProbeType.FORMAT, deep_probe)
+                for fpath in pathlist
+            ]
+            executor = concurrent.futures.ThreadPoolExecutor()
+            try:
+                durfutures = executor.map(lambda p: self.probe_val(*p), probe_args)
+            finally:
+                executor.shutdown(wait=False)
+        playlistpath = directory / name
+        with playlistpath.open(mode="x", encoding="utf-8") as f:
+            f.write("ffconcat version 1.0\n")
+            for fpath in pathlist:
+                f.write("file '" + str(fpath.resolve()).replace("'", "'\\''") + "'\n")
+                if durfutures is not None and (fdur := next(durfutures)) is not None:
+                    f.write(f"duration {fdur}\n")
+        return playlistpath
 
     @functools.cached_property
     def _header(self) -> FFBanner:
@@ -126,40 +339,30 @@ class FFBin:
             FFBanner: a named tuple of those properties.
         """
         ffversion = "0.0.0"
-        ffconfig = []
-        version_dict = collections.defaultdict(lambda: FFVersion(0, 0, 0))
-        versionargs = [self.ffmpeg, "-hide_banner", "-v", "0", "-version"]
-        result = subprocess.run(
-            versionargs,
-            capture_output=True,
-            env=self.env,
-            text=True,
-            encoding="utf-8",
-            check=False,
+        ffconfig: list[str] = []
+        version_dict: dict[str, FFVersion] = collections.defaultdict(
+            lambda: FFVersion(0, 0, 0)
         )
-        header_regex = re.compile(
-            r"""
-            ^\s*
-            (?:
-                ffmpeg\sversion\s*(?P<version>[^\s]*)
-                |
-                (?P<component>\w+)
-                [\s\d\.]*/
-                \s*(?P<major>\d+)\.\s*(?P<minor>\d+)\.\s*(?P<micro>\d+)
-                |
-                configuration:\s*(?P<config>.*)$
-            )
-            """,
-            flags=re.VERBOSE | re.MULTILINE,
+        jsonout = self.probe_json(
+            format_probe(
+                ("program_version", {"version", "configuration"}),
+                ("library_versions", ()),
+                allow_empty=True,
+            ),
+            None,
+            None,
         )
-        for match in header_regex.finditer(result.stdout):
-            if conf := match.group("config"):
-                ffconfig = conf.split(" ")
-            if ver := match.group("version"):
-                ffversion = ver
-            if component := match.group("component"):
-                version_dict[component] = FFVersion(
-                    match.group("major"), match.group("minor"), match.group("micro")
+        if jsonout is None:
+            return FFBanner(ffversion, ffconfig, version_dict)
+        with contextlib.suppress(KeyError):
+            ffversion = jsonout["program_version"]["version"]
+        with contextlib.suppress(KeyError):
+            ffconfig = jsonout["program_version"]["configuration"].split(" ")
+        libraries = jsonout.get("library_versions", [])
+        for library in libraries:
+            with contextlib.suppress(KeyError):
+                version_dict[library["name"]] = FFVersion(
+                    library["major"], library["minor"], library["micro"]
                 )
         return FFBanner(ffversion, ffconfig, version_dict)
 
@@ -186,7 +389,7 @@ class FFBin:
     @functools.cached_property
     def _encoders(self) -> FFEncoders:
         """All encoders compiled in."""
-        encoders: dict[str, set[str]] = {"V": set(), "A": set(), "S": set()}
+        encoders = {"V": set[str](), "A": set[str](), "S": set[str]()}
         encoderargs = [self.ffmpeg, "-hide_banner", "-v", "0", "-encoders"]
         result = subprocess.run(
             encoderargs,
@@ -230,7 +433,7 @@ class FFBin:
     @functools.cached_property
     def filters(self) -> set[str]:
         """Set of filters compiled into ffmpeg instance."""
-        filters = set()
+        filters = set[str]()
         versionargs = [self.ffmpeg, "-hide_banner", "-v", "0", "-filters"]
         result = subprocess.run(
             versionargs,
@@ -249,7 +452,7 @@ class FFBin:
             \s+
             (?P<filter>[\w-]+)
             \s*
-            (?P<src>[AVN|]+)->(?P<dest>[AVN|]+)
+            (?P<src>[AVSN|]+)->(?P<dest>[AVSN|]+)
             """,
             flags=re.VERBOSE | re.MULTILINE,
         )
@@ -261,7 +464,7 @@ class FFBin:
     @functools.cached_property
     def hwaccels(self) -> set[str]:
         """Set of hwaccels compiled into ffmpeg version."""
-        hwaccels = set()
+        hwaccels = set[str]()
         hwaccelargs = [self.ffmpeg, "-hide_banner", "-v", "0", "-hwaccels"]
         result = subprocess.run(
             hwaccelargs,
@@ -282,8 +485,8 @@ class FFBin:
     @functools.cached_property
     def protocols(self) -> FFProtocols:
         """A FFProtocols of compiled in input/output protocols."""
-        in_protocols: set[str] = set()
-        out_protocols: set[str] = set()
+        in_protocols = set[str]()
+        out_protocols = set[str]()
         protocol_args = [self.ffmpeg, "-hide_banner", "-v", "0", "-protocols"]
         result = subprocess.run(
             protocol_args,
@@ -314,228 +517,19 @@ class FFBin:
         return FFProtocols(in_protocols, out_protocols)
 
 
-ff_bin = FFBin("ffmpeg", "ffprobe", os.environ.copy())
+ff_bin = FFBin()
 
 
-class ProbeType(enum.Enum):
-    STREAM = enum.auto()
-    TAGS = enum.auto()
-    DISPOSITION = enum.auto()
-    FORMAT = enum.auto()
-    RAW_STREAM = enum.auto()
-    RAW_FORMAT = enum.auto()
-
-
-# TODO: use typing.TypeAlias in 3.10
-StrProbetype = Literal[
-    ProbeType.STREAM, ProbeType.TAGS, ProbeType.DISPOSITION, ProbeType.FORMAT
-]
-JsonProbetype = Literal[ProbeType.RAW_STREAM, ProbeType.RAW_FORMAT]
-StreamQueryTuple = tuple[Iterable[str], Iterable[str], Iterable[str]]
-# TODO: 3.10 | union syntax
-InitTuple = Union[StreamQueryTuple, Iterable[str]]
-
-
-class FFProbeJSON(TypedDict, total=False):
-    streams: Sequence[Mapping[str, Any]]
-    packets: Sequence[Mapping[str, Any]]
-    format: Mapping[str, str]
-
-
-_QUERY_PREFIX = {
-    ProbeType.STREAM: "stream=",
-    ProbeType.TAGS: "stream_tags=",
-    ProbeType.DISPOSITION: "stream_disposition=",
-    ProbeType.FORMAT: "format=",
-    ProbeType.RAW_STREAM: "",
-    ProbeType.RAW_FORMAT: "",
-}
-
-
-@overload
-def probe(
-    streamquery: str,
-    fileargs: str | Sequence[str],
-    streamtype: str,
-    probetype: StrProbetype,
-    deep_probe: bool = ...,
-    fallback: str | None = ...,
-    extraargs: str | Sequence[str] | None = ...,
-) -> str | None:
-    ...
-
-
-@overload
-def probe(
-    streamquery: str,
-    fileargs: str | Sequence[str],
-    streamtype: str,
-    probetype: JsonProbetype,
-    deep_probe: bool = ...,
-    fallback: str | None = ...,
-    extraargs: str | Sequence[str] | None = ...,
-) -> FFProbeJSON | None:
-    ...
-
-
-def probe(
-    streamquery: str,
-    fileargs: str | Sequence[str],
-    streamtype: str = "v",
-    probetype: ProbeType = ProbeType.STREAM,
-    deep_probe: bool = False,
-    fallback: str | None = None,
-    extraargs: str | Sequence[str] | None = None,
-) -> str | FFProbeJSON | None:
-    """Probes a media file with ffprobe and returns results.
-
-    Generic function for probing a media file for information using
-    ffmpeg's `ffprobe` utility. Capable of returning both individual
-    values and the raw deserialized JSON. Users may be interested in
-    querying many values at once with JSON ouput in order to avoid the
-    delays from repeatedly calling ffprobe.
-
-    Args:
-        streamquery:
-            Argument passed to the ``-show_entries`` flag in ffprobe. If
-            a non-raw streamtype is specified, then the argument may be
-            the type field you want to query, for example the duration.
-        fileargs:
-            String of the file you want to analyze. If additional args
-            are needed to specify the input, accepts a list of args to
-            pass on.
-        streamtype:
-            Optional; Argument to pass on to the ``-select_streams``
-            flag in ffprobe. Not needed if querying a format.
-        probetype:
-            Optional; If one of STREAM, TAGS, DISPOSITION, FORMAT, query
-            file for metadata of selected probetype and streamtype and
-            return the requested streamquery of the first matching
-            stream.
-            If one of RAW_STREAM, RAW_FORMAT, query file as before, but
-            return the raw JSON corresponding to the full streamquery
-            for all selected streamtype.
-        deep_probe:
-            Optional; Pass extra arguments to ffprobe in order to probe
-            the file more deeply. This is useful for containers that
-            can't be lightly inspected.
-        fallback:
-            Optional; Value to return if the query fails for any reason
-            or ffmpeg doesn't know the requested parameter value.
-        extraargs:
-            Optional; A list of additional arguments to past to ffprobe
-            during runtime. Can be used for example to request
-            ``-sexagesimal`` formatting of duration fields.
-
-    Returns:
-        fallback (default None): The query failed or returned "unknown"
-        or "N/A".
-
-        str: For non-raw probetype returns the value of the requested
-        query.
-
-        deserialized JSON: For raw probetype, the JSON returned after
-        deserialization.
-
-    Raises:
-        ValueError: Invalid probetype was passed.
-    """
-    if extraargs is None:
-        extraargs = []
-
-    # fmt: off
-    probeargs = [
-        ff_bin.ffprobe,
-        *(["-analyzeduration", "100M", "-probesize", "100M"] if deep_probe else []),
-        "-v", "0",
-        "-of", "json=c=1",
-        *((extraargs,) if isinstance(extraargs, str) else extraargs),
-        "-show_entries", _QUERY_PREFIX[probetype] + streamquery,
-        *((fileargs,) if isinstance(fileargs, str) else fileargs),
-    ]
-    # fmt: on
-    if probetype in {
-        ProbeType.STREAM,
-        ProbeType.TAGS,
-        ProbeType.DISPOSITION,
-        ProbeType.RAW_STREAM,
-    }:
-        probeargs += ["-select_streams", streamtype]
-    result = subprocess.run(
-        probeargs,
-        capture_output=True,
-        env=ff_bin.env,
-        text=True,
-        encoding="utf-8",
-        check=False,
+def format_probe(*queries: tuple[str, Iterable[str]], allow_empty: bool = False) -> str:
+    return ":".join(
+        f"{sect[0]}{'=' + ','.join(sect[1]) if sect[1] else ''}"
+        for sect in queries
+        if sect[1] or allow_empty
     )
-    if result.returncode != 0:
-        return fallback
-    jsonout: FFProbeJSON = json.loads(result.stdout)
-    if probetype in {ProbeType.RAW_STREAM, ProbeType.RAW_FORMAT}:
-        return jsonout
-    try:
-        # TODO: change to match case in 3.10
-        returnval: str
-        if probetype is ProbeType.STREAM:
-            returnval = jsonout["streams"][0][streamquery]
-        elif probetype is ProbeType.TAGS:
-            returnval = jsonout["streams"][0]["tags"][streamquery]
-        elif probetype is ProbeType.DISPOSITION:
-            returnval = jsonout["streams"][0]["disposition"][streamquery]
-        elif probetype is ProbeType.FORMAT:
-            returnval = jsonout["format"][streamquery]
-        else:
-            raise ValueError("invalid probe type query")
-    except (KeyError, IndexError):
-        # TODO: remove parentheses in 3.10
-        return fallback
-    return fallback if returnval in {"N/A", "unknown"} else returnval
 
 
-def make_playlist(
-    pathlist: Iterable[pathlib.Path],
-    directory: pathlib.Path,
-    add_duration: bool,
-    deep_probe: bool = False,
-    name: str = "streamplaylist.txt",
-) -> pathlib.Path:
-    """Construct ffconcat playlist from path list.
-
-    Paths created are absolute, and the files are probed in parallel if
-    needed to determine the duration.
-
-    Args:
-        pathlist: Ordered pathlist to construct ffconcat with.
-        directory: Directory to put the constructed playlist in.
-        add_duration: Whether or not to include duration metadata.
-        deep_probe: Whether to probe file deeply.
-        name: Optional; name of playlist file.
-
-    Returns:
-        Path to created ffconcat playlist.
-    """
-    durfutures = None
-    if add_duration:
-        iargs = [
-            ("duration", str(fpath), None, ProbeType.FORMAT, deep_probe)
-            for fpath in pathlist
-        ]
-        executor = concurrent.futures.ThreadPoolExecutor()
-        durfutures = executor.map(lambda p: probe(*p), iargs)
-        executor.shutdown(wait=False)
-    playlistpath = directory / name
-    with playlistpath.open(mode="x", newline="\n", encoding="utf-8") as f:
-        print("ffconcat version 1.0", file=f)
-        for fpath in pathlist:
-            print("file " + shlex.quote(str(fpath.resolve())), file=f)
-            if durfutures is not None and (fdur := next(durfutures)) is not None:
-                print("duration " + fdur, file=f)
-    return playlistpath
-
-
-def format_probestring(init_tuple: InitTuple | None, is_stream: bool) -> str:
-    """Format the streamquery arg for raw JSON queries to the probefile.
+def format_q_tuple(init_tuple: InitTuple | None, is_stream: bool) -> str:
+    """Format the entries arg for raw JSON queries to the probefile.
 
     This corresponds to the ``-show_entries`` flag in ffprobe and can be
     used generically to format arguments to it.
@@ -556,13 +550,12 @@ def format_probestring(init_tuple: InitTuple | None, is_stream: bool) -> str:
         return ""
     if is_stream:
         assert isinstance(init_tuple, tuple)
-        return_arr = ["stream=" + i for i in init_tuple[0]]
-        return_arr += ["stream_tags=" + i for i in init_tuple[1]]
-        return_arr += ["stream_disposition=" + i for i in init_tuple[2]]
+        return format_probe(
+            *zip(("stream", "stream_tags", "stream_disposition"), init_tuple)
+        )
     else:
         assert not isinstance(init_tuple, tuple)
-        return_arr = ["format=" + i for i in init_tuple]
-    return ":".join(return_arr)
+        return format_probe(("format", init_tuple))
 
 
 _SI_PREFIXES: Final[dict[str, float]] = {
@@ -590,7 +583,7 @@ _SI_PREFIXES: Final[dict[str, float]] = {
 }
 
 
-def num(val: str | int | float) -> float:
+def num(val: str) -> float:
     """Process input into float in a way that mimics ffmpeg.
 
     Method follows ffmpeg's `numerical options`_. All whitespace is
@@ -598,14 +591,13 @@ def num(val: str | int | float) -> float:
     SI prefix that may be appended with an `i` modifier that indicates
     the SI prefix is in powers of 1024 instead of 1000. Finally, the
     number may end in a `B` indicating it is to be multiplied by 8. The
-    optional ffmpeg utility ``ffeval`` may be used to validate the output
-    of this function.
+    optional ffmpeg utility ``ffeval`` may be used to validate the
+    output of this function.
 
     .. _numerical options:
        https://ffmpeg.org/ffmpeg.html#Options
     """
-    val = str(val)
-    val = re.sub(r"\s+", "", val)
+    val = re.sub(r"\s+", "", str(val))
     input_regex = re.compile(
         r"""
         (?P<number>-?(?:\d+\.?\d*|\.\d+))
@@ -617,7 +609,7 @@ def num(val: str | int | float) -> float:
         """,
         flags=re.VERBOSE | re.ASCII,
     )
-    if (match := input_regex.fullmatch(val)) is None:
+    if not (match := input_regex.fullmatch(val)):
         raise ValueError(f"Invalid ffmpeg number string: {val!r}")
     if not (basenum := match.group("number")):
         raise ValueError(f"Match number not found: {val!r}")
@@ -633,18 +625,13 @@ def duration(timestamp: str | float | int) -> float:
     https://ffmpeg.org/ffmpeg-utils.html#Time-duration
     """
     timestamp = str(timestamp)
-    if (
-        re.fullmatch(r"(\d+:)?\d?\d:\d\d(\.\d*)?", timestamp, flags=re.ASCII)
-        is not None
-    ):
+    if re.fullmatch(r"(\d+:)?\d?\d:\d\d(\.\d*)?", timestamp, flags=re.ASCII):
         return sum(
             s * float(t) for s, t in zip([1, 60, 3600], reversed(timestamp.split(":")))
         )
-    if (
-        match := re.fullmatch(
-            r"(-?(?:\d+\.?\d*|\.\d+))([mu]?s)?", timestamp, flags=re.ASCII
-        )
-    ) is not None:
+    if match := re.fullmatch(
+        r"(-?(?:\d+\.?\d*|\.\d+))([mu]?s)?", timestamp, flags=re.ASCII
+    ):
         val = float(match.group(1))
         if match.group(2) == "ms":
             val /= 1000
@@ -687,25 +674,16 @@ class Filter:
             self.basefilter = filt
             self.opts = []
             if filtopts:
-                if all(
-                    isinstance(list_, Sequence) and not isinstance(list_, str)
-                    for list_ in filtopts
-                ):
-                    for list_ in filtopts:
-                        self.opts += list_
-                else:
-                    self.opts = list(filtopts)
+                self.opts = list(filtopts)
 
     @staticmethod
     def escape_val(val: str) -> str:
-        chars = ":\\'"
-        trans = str.maketrans({char: "\\" + char for char in chars})
+        trans = str.maketrans({char: "\\" + char for char in ":\\'"})
         return val.translate(trans)
 
     @staticmethod
     def escape_graph(val: str) -> str:
-        chars = "\\'[],;"
-        trans = str.maketrans({char: "\\" + char for char in chars})
+        trans = str.maketrans({char: "\\" + char for char in "\\'[],;"})
         return val.translate(trans)
 
     @classmethod
@@ -724,6 +702,7 @@ class Filter:
         endkey: str = "b",
         basekey: str = "c",
     ) -> str:
+        """Combine filters in a way compatible with -filter_complex."""
         filts = copy.deepcopy(list(map(cls, filterlist)))
         filts[0].src = [key or startkey for key in filts[0].src]
         filts[-1].dst = [key or endkey for key in filts[-1].dst]
@@ -753,6 +732,7 @@ class Filter:
 
     @staticmethod
     def vf_join(filterlist: Sequence[str | Filter]) -> str:
+        """Combine filters in a way compatible with -af/-vf."""
         return ",".join(map(str, filterlist))
 
     # @classmethod
@@ -766,7 +746,7 @@ class Filter:
             return self.basefilter
 
 
-class Progress:
+class Progress(Generic[AnyStr]):
     """Assists in monitoring the progress output of an ffmpeg encode."""
 
     def __init__(self) -> None:
@@ -774,16 +754,15 @@ class Progress:
         self._sock.bind(("localhost", 0))
         self._sock.listen(1)
         self._packet_avail = threading.Event()
-        self._packet_lock = threading.Lock()
         self._progress_packet: list[str] = []
         self._loglevel: int
         self._futures: list[concurrent.futures.Future[Any]] = []
         self.progress_avail = threading.Event()
         self.finished = threading.Event()
-        self.output: collections.deque[str]
-        self.output_que: queue.Queue[str | None]
+        self.output: collections.deque[AnyStr]
+        self.output_que: queue.Queue[AnyStr | None]
         self._make_queue: bool
-        self.status: dict[str, str] = {
+        self.status = {
             "frame": "0",
             "fps": "0",
             "stream_0_0_q": "0.0",
@@ -821,8 +800,8 @@ class Progress:
 
     def monitor_progress(
         self,
-        result: subprocess.Popen[str],
-        outstream: typing.IO[str],
+        result: subprocess.Popen[AnyStr],
+        outstream: typing.IO[AnyStr],
         make_queue: bool = False,
         maxlen: int | None = None,
         loglevel: int = logging.DEBUG,
@@ -833,13 +812,17 @@ class Progress:
             self.output_que = queue.Queue()
         self.output = collections.deque(maxlen=maxlen)
         executor = concurrent.futures.ThreadPoolExecutor()
-        self._futures.append(executor.submit(self._read_outstream, result, outstream))
-        self._futures.append(executor.submit(self._read_progress, result))
-        self._futures.append(executor.submit(self._parse_progress))
-        executor.shutdown(wait=False)
+        try:
+            self._futures.append(
+                executor.submit(self._read_outstream, result, outstream)
+            )
+            self._futures.append(executor.submit(self._read_progress, result))
+            self._futures.append(executor.submit(self._parse_progress))
+        finally:
+            executor.shutdown(wait=False)
 
     def _read_outstream(
-        self, result: subprocess.Popen[str], outstream: typing.IO[str]
+        self, result: subprocess.Popen[AnyStr], outstream: typing.IO[AnyStr]
     ) -> None:
         try:
             while result.poll() is None:
@@ -855,7 +838,7 @@ class Progress:
             self._packet_avail.set()
             self.progress_avail.set()
 
-    def _parse_outputline(self, line: str) -> None:
+    def _parse_outputline(self, line: AnyStr) -> None:
         if not line:
             return
         logger.log(self._loglevel, line)
@@ -863,28 +846,29 @@ class Progress:
         if self._make_queue:
             self.output_que.put(line)
 
-    def _read_progress(self, result: subprocess.Popen[str]) -> None:
+    def _read_progress(self, result: subprocess.Popen[AnyStr]) -> None:
         conn, _ = self._sock.accept()
-        with contextlib.closing(conn), contextlib.closing(self._sock), conn.makefile(
-            encoding="utf-8"
-        ) as f:
+        # https://github.com/python/mypy/issues/9743
+        f: typing.IO[str]
+        # TODO: 3.10 parenthesized context managers
+        with contextlib.suppress(ConnectionResetError), contextlib.closing(
+            conn
+        ), contextlib.closing(self._sock), conn.makefile(encoding="utf-8") as f:
             while result.poll() is None:
-                packet = []
+                packet: list[str] = []
                 while line := f.readline():
                     if line.startswith("progress="):
                         break
                     if line := line.rstrip():
                         packet.append(line)
                 if packet:
-                    with self._packet_lock:
-                        self._progress_packet = packet
+                    self._progress_packet = packet
                     self._packet_avail.set()
 
     def _parse_progress(self) -> None:
         self._packet_avail.wait()
         while not self.finished.is_set():
-            with self._packet_lock:
-                packet = self._progress_packet.copy()
+            packet = self._progress_packet
             for line in packet:
                 split = line.split("=", 1)
                 try:
@@ -899,5 +883,7 @@ class Progress:
         self._sock.close()
         concurrent.futures.wait(self._futures)
         for future in self._futures:
-            if exception := future.exception(5):
-                raise exception
+            # https://github.com/python/mypy/issues/9743
+            exception = future.exception(5)
+            if exception:
+                logger.error("unexpected exception: %r", exception)
